@@ -1,11 +1,13 @@
 /*
  * Background display property module.
  * (C) 1997 the Free Software Foundation
+ * (C) 2000 Helix Code, Inc.
  *
  * Authors: Miguel de Icaza.
  *          Federico Mena.
  *          Radek Doulik
  *          Michael Fulbright
+ *          Justin Maurer
  */
 
 #include <config.h>
@@ -26,6 +28,7 @@ static GtkWidget *preview;
 #endif
 
 static GtkWidget *monitor;
+static GtkWidget *menuitem1, *menuitem2, *menuitem3;
 
 static GtkWidget *fileSel = NULL;
 static GtkWidget *wpMenu;
@@ -197,7 +200,7 @@ fill_monitor (int prop_changed, struct bgState *state)
      * if we are called from the session manager, though
      */
     
-    if (capplet) {
+    if (capplet && capplet->window) {
 	cursor = gdk_cursor_new (GDK_WATCH);
 	gdk_window_set_cursor (capplet->window, cursor);
 	gdk_cursor_destroy (cursor);
@@ -275,7 +278,7 @@ fill_monitor (int prop_changed, struct bgState *state)
 	    gtk_widget_queue_draw (monitor);
 	}
 
-	if (capplet)
+	if (capplet && capplet->window)
 	    gdk_window_set_cursor (capplet->window, NULL);
 	
 	return;
@@ -538,68 +541,27 @@ idle_fill_monitor (gpointer data)
 	return FALSE;
 }
 
-static GtkWidget *radioh, *radiov, *radiof, *radiog, *radion;
+static GtkWidget *radion;
 static GtkWidget *tiledButton, *scaledButton;
 static GtkWidget *wallp, *fill;
 static GtkWidget *scaledkeepButton, *centeredButton;
 
 static void
-set_background_mode (GtkWidget *widget)
+set_enabled (GtkWidget *widget)
 {
-    if (GTK_TOGGLE_BUTTON (widget)->active) {
-        if (widget == radion) {
-	    /* Disabled */
-	    gtk_widget_set_sensitive(wallp, FALSE);
-	    gtk_widget_set_sensitive(fill, FALSE);
-	    curState.enabled = FALSE;
-        } else if (widget == radiof) {
-	    /* Flat color */
-	    gtk_widget_set_sensitive(cp1, TRUE);
-	    gtk_widget_set_sensitive(cp2, FALSE);
-	    gtk_widget_set_sensitive(cl2, FALSE);
-	    gtk_widget_set_sensitive(radioh, FALSE);
-	    gtk_widget_set_sensitive(radiov, FALSE);
-	    curState.enabled = TRUE;
-	    curState.grad = FALSE;
-	} else if (widget == radiog) {
-	    /* Gradient fill */
-	    gtk_widget_set_sensitive(cp1, TRUE);
-	    gtk_widget_set_sensitive(cp2, TRUE);
-	    gtk_widget_set_sensitive(cl2, TRUE);
-	    gtk_widget_set_sensitive(radioh, TRUE);
-	    gtk_widget_set_sensitive(radiov, TRUE);
-	    curState.enabled = TRUE;
-	    curState.grad = TRUE;
-	}
-
-	fill_monitor (!ignoreChanges, &curState);
-    } else if (widget == radion) {
-	    gtk_widget_set_sensitive(wallp, TRUE);
-	    gtk_widget_set_sensitive(fill, TRUE);
-	    curState.enabled = TRUE;
-	    fill_monitor (!ignoreChanges, &curState);
-    }	    
-    if (!ignoreChanges)
-	capplet_widget_state_changed(CAPPLET_WIDGET(capplet), TRUE);
-
-}
-
-static void
-set_orientation (GtkWidget *widget)
-{
-    if (GTK_TOGGLE_BUTTON (widget)->active) {
-	if (widget == radioh)
-	    curState.vertical = FALSE;  /* Horizontal gradient */
-	else if (widget == radiov)
-	    curState.vertical = TRUE;   /* Vertical gradient */
-	
-	fill_monitor (!ignoreChanges, &curState);
+    if (!GTK_TOGGLE_BUTTON (widget)->active) {
+        /* Disabled */
+        gtk_widget_set_sensitive(wallp, FALSE);
+        gtk_widget_set_sensitive(fill, FALSE);
+        curState.enabled = FALSE;
+    } else {
+	gtk_widget_set_sensitive(wallp, TRUE);
+	gtk_widget_set_sensitive(fill, TRUE);
+        curState.enabled = TRUE;
     }
-    if (!ignoreChanges)
-	capplet_widget_state_changed(CAPPLET_WIDGET(capplet), TRUE);
-
+    fill_monitor (!ignoreChanges, &curState); 
 }
-
+    
 static void
 set_wallpaper_type (GtkWidget *widget, gpointer data)
 {
@@ -612,22 +574,83 @@ set_wallpaper_type (GtkWidget *widget, gpointer data)
 	capplet_widget_state_changed(CAPPLET_WIDGET(capplet), TRUE);
 }
 
+static void
+set_solid (GtkWidget *widget, gpointer data)
+{
+    gtk_widget_set_sensitive(cp1, TRUE);
+    gtk_widget_set_sensitive(cp2, FALSE);
+    gtk_widget_set_sensitive(cl2, FALSE);
+    curState.enabled = TRUE;
+    curState.grad = FALSE;
+    fill_monitor (!ignoreChanges, &curState);
+}
+
+static void
+set_grad (GtkWidget *widget, gpointer data)
+{
+    gtk_widget_set_sensitive(cp1, TRUE);
+    gtk_widget_set_sensitive(cp2, TRUE);
+    gtk_widget_set_sensitive(cl2, TRUE);
+    curState.enabled = TRUE;
+    curState.grad = TRUE;
+    if ((gboolean)data == TRUE)
+        curState.vertical = FALSE;
+    else
+        curState.vertical = TRUE;
+    fill_monitor (!ignoreChanges, &curState);
+}
+
 static GtkWidget *
 color_setup (struct bgState *state)
 {
     GtkWidget *frame;
     GtkWidget *table;
+    GtkWidget *menu, *optionmenu;
     
     frame = gtk_frame_new (_("Color"));
     
-    
-    table = gtk_table_new (3, 5, FALSE);
+    table = gtk_table_new (3, 2, FALSE);
     gtk_container_set_border_width (GTK_CONTAINER(table), GNOME_PAD);
     gtk_table_set_col_spacings (GTK_TABLE(table), GNOME_PAD);
     gtk_container_add (GTK_CONTAINER(frame), table);
     
+    menu = gtk_menu_new ();
+
+    menuitem1 = gtk_menu_item_new_with_label(_("Solid"));
+    gtk_signal_connect(GTK_OBJECT(menuitem1), "activate", GTK_SIGNAL_FUNC(set_solid), 
+                       NULL);
+    gtk_menu_append(GTK_MENU(menu), menuitem1); 
+    gtk_widget_show(menuitem1);
+
+    menuitem2 = gtk_menu_item_new_with_label(_("Horizontal Gradient"));
+    gtk_signal_connect(GTK_OBJECT(menuitem2), "activate", GTK_SIGNAL_FUNC(set_grad), 
+                       GINT_TO_POINTER (TRUE));
+    gtk_menu_append(GTK_MENU(menu), menuitem2);
+    gtk_widget_show(menuitem2);
+
+    menuitem3 = gtk_menu_item_new_with_label(_("Vertical Gradient"));
+    gtk_signal_connect(GTK_OBJECT(menuitem3), "activate", GTK_SIGNAL_FUNC(set_grad),
+		       FALSE);
+    gtk_menu_append(GTK_MENU(menu), menuitem3);
+    gtk_widget_show(menuitem3);    
+
+    optionmenu = gtk_option_menu_new();
+    gtk_option_menu_set_menu(GTK_OPTION_MENU(optionmenu), menu);
+
+    if (curState.grad) {
+	if (curState.vertical) {
+		gtk_option_menu_set_history (GTK_OPTION_MENU(optionmenu), 2);
+	} else {
+		gtk_option_menu_set_history (GTK_OPTION_MENU(optionmenu), 1);
+	}
+    } else {
+   	gtk_option_menu_set_history (GTK_OPTION_MENU(optionmenu), 0);
+    }
+
+    gtk_table_attach (GTK_TABLE(table), optionmenu, 0, 2, 0, 1,
+		      GTK_EXPAND | GTK_SHRINK | GTK_FILL,
+		      GTK_EXPAND | GTK_SHRINK, 0, 0); 
 
-    /* The left side */
     cp1 = gnome_color_picker_new ();
     gnome_color_picker_set_i16 (GNOME_COLOR_PICKER(cp1), 
 				state->bgColor1.red, state->bgColor1.green,
@@ -641,55 +664,17 @@ color_setup (struct bgState *state)
 				state->bgColor2.blue, 0xff);
     gtk_signal_connect(GTK_OBJECT(cp2), "color_set", 
 		       GTK_SIGNAL_FUNC(color_sel_set), NULL);
-    gtk_table_attach_defaults (GTK_TABLE(table), gtk_label_new (_("Color 1")), 0, 1, 0, 1);
-    gtk_table_attach (GTK_TABLE(table), cp1, 0, 1, 1, 2, 0, 0, 0, 0);
-/*    gtk_table_attach_defaults (GTK_TABLE(table), cp1, 0, 1, 1, 2);*/
-/*    gtk_table_attach_defaults (GTK_TABLE(table), gtk_hseparator_new (), 0, 1, 2, 3);*/
-    cl2 = gtk_label_new (_("Color 2"));
-    gtk_table_attach_defaults (GTK_TABLE(table), cl2, 0, 1, 3, 4);
-    gtk_table_attach (GTK_TABLE(table), cp2, 0, 1, 4, 5, 0, 0, 0, 0);
-
-    /* The middle side */
-    gtk_table_attach_defaults (GTK_TABLE(table), gtk_vseparator_new (), 1, 2, 0, 5);
-    
-    /* The right side */
-    radiof = gtk_radio_button_new_with_label (NULL, 
-					      _("Solid"));
-    radiog = gtk_radio_button_new_with_label (gtk_radio_button_group(GTK_RADIO_BUTTON(radiof)),
-					      _("Gradient"));
-    radiov = gtk_radio_button_new_with_label (NULL, _("Vertical"));
-    radioh = gtk_radio_button_new_with_label (gtk_radio_button_group(GTK_RADIO_BUTTON(radiov)),
-					      _("Horizontal"));
-    gtk_table_attach_defaults (GTK_TABLE(table), radiof, 2, 3, 0, 1);
-    gtk_table_attach_defaults (GTK_TABLE(table), radiog, 2, 3, 1, 2);
-    gtk_table_attach_defaults (GTK_TABLE(table), gtk_hseparator_new (), 2, 3, 2, 3);
-    gtk_table_attach_defaults (GTK_TABLE(table), radiov, 2, 3, 3, 4);
-    gtk_table_attach_defaults (GTK_TABLE(table), radioh, 2, 3, 4, 5);
-
-    /* BEFORE our signals are connected, set initial states */
-    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON((state->grad) ? radiog : radiof), TRUE);
-    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON((state->vertical) ? radiov : radioh), TRUE);
+    gtk_table_attach_defaults (GTK_TABLE(table), gtk_label_new (_("Primary Color")), 0, 1, 1, 2);
+    gtk_table_attach (GTK_TABLE(table), cp1, 1, 2, 1, 2, 0, 0, 0, 0);
+    cl2 = gtk_label_new (_("Secondary Color"));
+    gtk_table_attach_defaults (GTK_TABLE(table), cl2, 0, 1, 2, 3);
+    gtk_table_attach (GTK_TABLE(table), cp2, 1, 2, 2, 3, 0, 0, 0, 0);
 
     if (!state->grad) {
 	    gtk_widget_set_sensitive (cp2, FALSE);
 	    gtk_widget_set_sensitive (cl2, FALSE);
-	    gtk_widget_set_sensitive (radioh, FALSE);
-	    gtk_widget_set_sensitive (radiov, FALSE);
     }
 
-    /* When people change the background mode controls, update */
-    gtk_signal_connect (GTK_OBJECT(radiog), "toggled",
-			GTK_SIGNAL_FUNC(set_background_mode),
-			NULL);
-    gtk_signal_connect (GTK_OBJECT(radiof), "toggled",
-			(GtkSignalFunc) set_background_mode,
-			NULL);    
-    gtk_signal_connect (GTK_OBJECT(radiov), "toggled",
-			(GtkSignalFunc) set_orientation,
-			(gpointer) ((long) TRUE));
-    gtk_signal_connect (GTK_OBJECT(radioh), "toggled",
-			(GtkSignalFunc) set_orientation,
-			(gpointer) ((long) FALSE));
     gtk_idle_add ((GtkFunction) idle_fill_monitor, NULL);
     gtk_widget_show_all (frame);
     return frame;
@@ -713,14 +698,7 @@ browse_activated (GtkWidget *w, gchar *s)
 		g_free(curState.wpFileName);
 	curState.wpFileName = g_strdup(s);
 	curState.bgType = (s) ? BACKGROUND_WALLPAPER : BACKGROUND_SIMPLE;
-	/* printf ("%s\n", s); */
-/*
-	if (curState.bgType == BACKGROUND_WALLPAPER) {
-	    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radiof), TRUE);
-	}
-*/
 	fill_monitor (TRUE, &curState);
-	/* printf ("%s\n", __FUNCTION__); */
 }
 
 static void
@@ -1067,22 +1045,14 @@ background_apply (struct bgState *state)
 				state->bgColor2.blue, 0xff);
     if (!state->enabled)
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radion), TRUE);
-    else
-	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON((state->grad) ? radiog : radiof), TRUE);
-
-    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON((state->vertical) ? radiov : radioh), TRUE);
 
     if (state->enabled) {
 	gtk_widget_set_sensitive(cp2, state->grad);
 	gtk_widget_set_sensitive(cl2, state->grad);
-	gtk_widget_set_sensitive(radioh, state->grad);
-	gtk_widget_set_sensitive(radiov, state->grad);
     } else {
 	gtk_widget_set_sensitive(cp1, FALSE);
 	gtk_widget_set_sensitive(cp2, FALSE);
 	gtk_widget_set_sensitive(cl2, FALSE);
-	gtk_widget_set_sensitive(radioh, FALSE);
-	gtk_widget_set_sensitive(radiov, FALSE);
     }
     switch (state->wpType) {
       case WALLPAPER_SCALED:
@@ -1320,59 +1290,40 @@ background_setup (struct bgState *state)
     };
     static gint n_drop_types = sizeof (drop_types) / sizeof(drop_types[0]);
 
-    GtkWidget *settings;
-    GtkWidget *vbox, *hbox;
-    GtkWidget *align, *align2;
-
+    GtkWidget *frame;
+    GtkWidget *vbox, *vbox2;
+    GtkWidget *hbox;
+    
     capplet = capplet_widget_new();
+    vbox = gtk_vbox_new (FALSE, GNOME_PAD_SMALL);
+    vbox2 = gtk_vbox_new (FALSE, GNOME_PAD_SMALL);
+    hbox = gtk_hbox_new (FALSE, GNOME_PAD_SMALL);
 
-    vbox = gtk_vbox_new (FALSE, 0);
-    hbox = gtk_hbox_new (FALSE, 0);
-    gtk_container_set_border_width (GTK_CONTAINER(hbox), GNOME_PAD);
-
-    align  = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
-    align2 = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
 
+    wallp  = wallpaper_setup (state);
+    fill = color_setup (state);
     monitor = get_monitor_preview_widget ();
-
+    frame = gtk_frame_new (_("Preview") );
+    gtk_container_add (GTK_CONTAINER(frame), monitor);
     gtk_drag_dest_set (GTK_WIDGET (monitor),
 		       GTK_DEST_DEFAULT_MOTION |
 		       GTK_DEST_DEFAULT_HIGHLIGHT |
 		       GTK_DEST_DEFAULT_DROP,
 		       drop_types, n_drop_types,
 		       GDK_ACTION_COPY);
-
     gtk_signal_connect (GTK_OBJECT (monitor), "drag_data_received",
 			GTK_SIGNAL_FUNC (img_dnd_drop), NULL);
-			    
     gdk_null_window_warnings = 0;
-#if 0
-    preview = gtk_preview_new(GTK_PREVIEW_COLOR);
-    gtk_preview_size(GTK_PREVIEW(preview),
-		     MONITOR_CONTENTS_WIDTH,
-		     MONITOR_CONTENTS_HEIGHT);
-#endif
-    gtk_container_add (GTK_CONTAINER (align), monitor);
-    gtk_box_pack_start (GTK_BOX(hbox), align, TRUE, TRUE, 0); 
-    gtk_container_add (GTK_CONTAINER (align2), hbox);
 
-    gtk_box_pack_start (GTK_BOX (vbox), align2, TRUE, TRUE, 0);
-
-    	
-    radion = gtk_check_button_new_with_label ( _("Disable background selection") );
-    gtk_widget_show (radion);
-    gtk_box_pack_end (GTK_BOX (vbox), radion, FALSE, FALSE, 0);
-
-    settings = gtk_hbox_new (FALSE, GNOME_PAD);
-    gtk_container_set_border_width (GTK_CONTAINER(settings), GNOME_PAD);
-    gtk_box_pack_end (GTK_BOX (vbox), settings, FALSE, FALSE, 0);
-
-    fill = color_setup (state);
-    gtk_box_pack_start (GTK_BOX (settings), fill, FALSE, FALSE, 0);
-	
-    wallp  = wallpaper_setup (state);
-    gtk_box_pack_end (GTK_BOX (settings), wallp, TRUE, TRUE, 0);
+    radion = gtk_check_button_new_with_label ( _("Use GNOME to set background") );
 
+    gtk_box_pack_start (GTK_BOX (vbox2), wallp, TRUE, TRUE, 0);
+    gtk_box_pack_start (GTK_BOX (vbox2), fill, TRUE, TRUE, 0);
+    gtk_box_pack_start (GTK_BOX (hbox), vbox2, TRUE, TRUE, 0);
+    gtk_box_pack_start (GTK_BOX (hbox), frame, TRUE, TRUE, 0);
+    gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 0);
+    gtk_box_pack_start (GTK_BOX (vbox), radion, FALSE, FALSE, 0);
+    
     if (!state->enabled) {
 	    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radion), TRUE);
 	    gtk_widget_set_sensitive (fill, FALSE);
@@ -1380,16 +1331,10 @@ background_setup (struct bgState *state)
     }
 
     gtk_signal_connect (GTK_OBJECT(radion), "toggled",
-			(GtkSignalFunc) set_background_mode,
+			(GtkSignalFunc) set_enabled,
 			NULL);
 
-    gtk_widget_show (align);
-    gtk_widget_show (align2);
-    gtk_widget_show (monitor);
-    gtk_widget_show (settings);
-    gtk_widget_show (hbox);
-    gtk_widget_show (vbox);
-
+    gtk_widget_show_all (vbox);
 
     gtk_signal_connect (GTK_OBJECT (capplet), "help",
 			GTK_SIGNAL_FUNC (background_help), &curState);
@@ -1403,6 +1348,7 @@ background_setup (struct bgState *state)
 			GTK_SIGNAL_FUNC (background_ok), &curState);
 
     gtk_container_add(GTK_CONTAINER(capplet), vbox);
+    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radion), TRUE);
     gtk_widget_show(capplet);
 }
 
