@@ -129,8 +129,9 @@ static WidgetLayout menu_options[] = {
 	 gnome_preferences_set_menus_have_tearoff,
 	 NULL},
 	{menus_have_icons_name, NULL,
-	 NULL, NULL,
-	 "/Gnome/Icons/MenusUseIcons=true"},
+	 gnome_preferences_get_menus_have_icons,
+	 gnome_preferences_set_menus_have_icons,
+	 NULL},
 	{NULL, NULL, NULL, NULL, NULL}
 };
 
