@@ -211,18 +211,19 @@ generate_tree_helper (GtkCTree *ctree, GtkCTreeNode *parent, GNode *node)
                         text[0] = ((GnomeDesktopEntry *)i->data)->name;
                 else
                         text[0] = "*MISSINGNAME*";
-#if 0
+
                 if ((i->data) && ((GnomeDesktopEntry *)i->data)->icon)
                         icon = ((GnomeDesktopEntry *)i->data)->icon;
                 if (icon && g_file_exists (icon))
-                        icon_gpixmap = (GnomePixmap *)gnome_pixmap_new_from_file_at_size(icon,
-                                                                                         SMALL_ICON_SIZE,
-                                                                                         SMALL_ICON_SIZE);
+                        icon_gpixmap = (GnomePixmap *)gnome_stock_pixmap_widget_at_size(NULL,
+                                                                                        icon,
+                                                                                        SMALL_ICON_SIZE,
+                                                                                        SMALL_ICON_SIZE);
                 if (icon_gpixmap) {
                         icon_pixmap = icon_gpixmap->pixmap;
                         icon_mask   = icon_gpixmap->mask;
                 }
-#endif
+
                 if (i->data && (!strcmp(((GnomeDesktopEntry *)i->data)->type,"Directory")))
                         child = gtk_ctree_insert_node (ctree,parent,NULL, text, 3,
                                                        icon_pixmap, icon_mask,
@@ -238,7 +239,7 @@ generate_tree_helper (GtkCTree *ctree, GtkCTreeNode *parent, GNode *node)
                 data->id = -1;
                 data->capplet = NULL;
                 data->ctree = ctree;
-                data->notetab_id = -1;
+                data->child = NULL;
                 data->modified = FALSE;
                 data->state = CAPPLET_INACTIVE;
                 gtk_ctree_node_set_row_data (ctree, child, data);
@@ -270,7 +271,7 @@ generate_tree_helper (GtkCTree *ctree, GtkCTreeNode *parent, GNode *node)
 
 
 GtkWidget *
-generate_tree ()
+generate_tree (void)
 {
         GtkWidget *retval;
         GNode *global_node;
@@ -293,7 +294,6 @@ generate_tree ()
 
         gtk_ctree_set_indent (GTK_CTREE (retval), 15);
         gtk_clist_set_column_auto_resize (GTK_CLIST (retval), 0, TRUE);
-        /*gtk_widget_set_usize (retval, 200, 375);*/
         gtk_clist_set_selection_mode(GTK_CLIST(retval), GTK_SELECTION_SINGLE );
         gtk_signal_connect( GTK_OBJECT (retval),"tree_select_row", GTK_SIGNAL_FUNC (selected_row_callback), NULL);
         gtk_signal_connect( GTK_OBJECT (retval),"unselect_row", GTK_SIGNAL_FUNC (unselected_row_callback), NULL);
